# $Id: tcom.tcl,v 1.4 2000/05/09 15:03:10 chuang Exp $

namespace eval ::tcom {
    # registry key for storing server implementation data
    variable classKey \
	"HKEY_LOCAL_MACHINE\\Software\\Scriptics\\Tcl\\tcom\\CLSID"

    # This command is deprecated.  Use search_auto_path instead.
    proc findfile {subDirName fileName} {
	global auto_path

	foreach dir [set auto_path] {
	    set filePath [file join $dir $subDirName $fileName]
	    if {[file exists $filePath]} {
		return [file nativename $filePath]
	    }
	}
	error "cannot find $fileName"
    }

    # Look for the file in the directories in the package load path.
    # Return the file path.
    proc search_auto_path {fileSpec} {
	global auto_path

	foreach dir [set auto_path] {
	    set filePath [file join $dir $fileSpec]
	    if {[file exists $filePath]} {
		return [file nativename $filePath]
	    }
	}
	error "cannot find $fileSpec"
    }

    # Insert registry entries for the class.
    proc registerClass {typeLibName className clsid} {
	set dllPath [findfile "tcom" "tcomserver.dll"]
	set exePath [findfile "tcom" "tcomserver.exe"]
	set verIndependentProgId "$typeLibName.$className"
	set progId "$verIndependentProgId.1"

	set key "HKEY_CLASSES_ROOT\\$progId"
	registry set $key "" "$className Class"
	registry set "$key\\CLSID" "" $clsid

	set key "HKEY_CLASSES_ROOT\\$verIndependentProgId"
	registry set $key "" "$className Class"
	registry set "$key\\CLSID" "" $clsid

	set key "HKEY_CLASSES_ROOT\\CLSID\\$clsid"
	registry set $key "" "$className Class"
	registry set "$key\\ProgID" "" $progId
	registry set "$key\\VersionIndependentProgID" "" $verIndependentProgId

	set key "HKEY_CLASSES_ROOT\\CLSID\\$clsid\\InprocServer32"
	registry set $key "" $dllPath
	registry set $key "ThreadingModel" "Both"

	set key "HKEY_CLASSES_ROOT\\CLSID\\$clsid\\LocalServer32"
	registry set $key "" $exePath
	registry set $key "ThreadingModel" "Both"

	set key "$::tcom::classKey\\$clsid"
	registry set $key "" "$className Class"
	registry set $key "Script" "package require $typeLibName"
    }

    # Remove registry entries for the class.
    proc unregisterClass {typeLibName className clsid} {
	set verIndependentProgId "$typeLibName.$className"
	set progId "$verIndependentProgId.1"

	registry delete "HKEY_CLASSES_ROOT\\$progId"
	registry delete "HKEY_CLASSES_ROOT\\$verIndependentProgId"
	registry delete "HKEY_CLASSES_ROOT\\CLSID\\$clsid"
	registry delete "$::tcom::classKey\\$clsid"
    }

    # Register or unregister servers for classes defined in a type library.
    proc server {option typeLibFile args} {
	package require registry

	switch -- $option {
	    register {
		::tcom::typelib register $typeLibFile
		set registerOpt 1
	    }
	    unregister {
		set registerOpt 0
	    }
	    default {
		error "bad option $option: must be register or unregsiter"
	    }
	}

	set typeLib [::tcom::typelib load $typeLibFile]
	set typeLibName [$typeLib name]

	if {[llength $args] == 0} {
	    set classes [$typeLib class]
	} else {
	    set classes $args
	}

	foreach className $classes {
	    set classInfo [$typeLib class $className] 
	    set clsid "{[string toupper [lindex $classInfo 0]]}"
	    if {$registerOpt} {
		registerClass $typeLibName $className $clsid 
	    } else {
		unregisterClass $typeLibName $className $clsid 
	    }
	}

	::tcom::release $typeLib
    }
}
