// $Id: Arguments.h,v 1.4 1999/12/10 22:29:20 chuang Exp $
#ifndef ARGUMENTS_H
#define ARGUMENTS_H
#pragma warning(disable: 4786)

#include "TypeInfo.h"

// This abstract class represents the values passed in and out of a method
// as arguments.

class Arguments
{
protected:
    DISPPARAMS m_dispParams;

    // argument values
    _variant_t *m_args;

    // used to hold values returned from out parameters
    _variant_t *m_outValues;

    Arguments();

    // Initialize a single argument.
    int initArgument(
        Tcl_Interp *interp,
        Tcl_Obj *obj,
        int argIndex,
        const Parameter &parameter);

public:
    virtual ~Arguments();

    // Ready arguments for method invocation.
    // Returns a Tcl result code.
    virtual int init(
        Tcl_Interp *interp,
        int objc,
        Tcl_Obj *CONST objv[],
        const Method &method,
        INVOKEKIND invokeKind) = 0;

    // Get arguments in the format required by the Invoke function.
    DISPPARAMS *dispParams () const
    { return const_cast<DISPPARAMS *>(&m_dispParams); }

    // Put the values returned from out parameters into Tcl variables.
    void storeOutValues(
        Tcl_Interp *interp,
        int objc,
        Tcl_Obj *CONST objv[],
        const Method::Parameters &parameters);
};

// This class represents arguments specified by their position in an argument
// list.

class PositionalArguments: public Arguments
{
public:
    PositionalArguments ()
    { }

    // Ready arguments for method invocation.
    // Returns a Tcl result code.
    virtual int init(
        Tcl_Interp *interp,
        int objc,
        Tcl_Obj *CONST objv[],
        const Method &method,
        INVOKEKIND invokeKind);
};

// This class represents arguments specified by name.

class NamedArguments: public Arguments
{
    // Search the parameter list for the named parameter.
    static Method::Parameters::const_iterator findParameter(
        const Method::Parameters &parameters,
        const std::string &name,
        DISPID &dispid);

    // IDs of named arguments
    DISPID *m_namedDispids;

public:
    NamedArguments ():
        m_namedDispids(0)
    { }

    ~NamedArguments();

    // Ready arguments for method invocation.
    // Returns a Tcl result code.
    virtual int init(
        Tcl_Interp *interp,
        int objc,
        Tcl_Obj *CONST objv[],
        const Method &method,
        INVOKEKIND invokeKind);
};

#endif 
