// $Id: ComObjectFactory.h,v 1.4 2000/04/26 16:43:19 chuang Exp $
#ifndef COMOBJECTFACTORY_H
#define COMOBJECTFACTORY_H

#include "tcomApi.h"
#include "mutex.h"
#include "TclObject.h"
#include "TypeInfo.h"

// This is a factory of COM objects.

class TCOM_API ComObjectFactory: public IClassFactory
{
    // reference count of the factory
    long m_refCount;

    // description of interface to implement
    const Interface &m_interfaceDesc;

    // thread running the Tcl interpreter to use
    Tcl_ThreadId m_threadId;
                                            
    // Tcl interpreter used to execute Tcl commands
    Tcl_Interp *m_interp;

    // Tcl command executed to create a servant
    TclObject m_constructor;

    // Tcl command executed to destroy servant
    TclObject m_destructor;

    // handle of registered class object
    unsigned long m_classObjectHandle;

public:
    ComObjectFactory(
        const Interface &interfaceDesc,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject constructor,
        TclObject destructor);

    // Register factory.
    void registerFactory(REFCLSID clsid);

    // Revoke factory.
    void revokeFactory();

    // IUnknown methods
    STDMETHOD(QueryInterface)(REFIID riid, void **ppvObj);
    STDMETHOD_(ULONG, AddRef)();
    STDMETHOD_(ULONG, Release)();

    // IClassFactory methods
    STDMETHOD(CreateInstance)(IUnknown *pOuter, REFIID riid, void **ppvObj);
    STDMETHOD(LockServer)(BOOL fLock);
};

// This is factory returns always the same instance.

class TCOM_API SingletonObjectFactory: public ComObjectFactory
{
    DECLARE_MUTEX(m_mutex)

    IUnknown *m_pInstance;

public:
    SingletonObjectFactory(
        const Interface &interfaceDesc,
        Tcl_ThreadId threadId,
        Tcl_Interp *interp,
        TclObject constructor,
        TclObject destructor);

    // Override create function.
    STDMETHOD(CreateInstance)(IUnknown *pOuter, REFIID riid, void **ppvObj);
};

#endif 
