// $Id: HashTable.h,v 1.9 2000/04/28 19:40:06 chuang Exp $
#ifndef HASHTABLE_H
#define HASHTABLE_H

#include <tcl.h>
#include "mutex.h"

// This is class wraps a Tcl hash table.

template<class K, class D>
class HashTable
{
    DECLARE_MUTEX(m_mutex)
    Tcl_HashTable m_hashTable;

public:
    typedef K key_type;
    typedef D referent_type;

    HashTable();
    ~HashTable();

    // Insert value into table.
    void insert(const K &key, D pValue);

    // Find value in table.
    D find(const K &key) const;

    // Remove value.
    void erase(const K &key);
};

template<class K, class D>
HashTable<K,D>::HashTable ()
{
    Tcl_InitHashTable(&m_hashTable, sizeof(K)/sizeof(int));
}

template<class K, class D>
HashTable<K,D>::~HashTable ()
{
    // TODO: I would like to clean up resources here, but when the destructor
    // is called on static instances of this class, the Tcl memory subsystem
    // may have been finalized, making the argument point to an invalid memory
    // address.
//    Tcl_DeleteHashTable(&m_hashTable);
}

// Insert a key and value.  If the key already exists in the table,
// replace the existing value.

template<class K, class D>
void
HashTable<K,D>::insert (const K &key, D pValue)
{
    LOCK_MUTEX(m_mutex)

    int isNew;
    Tcl_HashEntry *pEntry = Tcl_CreateHashEntry(
        &m_hashTable,
        reinterpret_cast<const char *>(&key),
        &isNew);
    Tcl_SetHashValue(pEntry, reinterpret_cast<ClientData>(pValue));
}

template<class K, class D>
D 
HashTable<K,D>::find (const K &key) const
{
    LOCK_MUTEX(m_mutex)

    Tcl_HashEntry *pEntry = Tcl_FindHashEntry(
        const_cast<Tcl_HashTable *>(&m_hashTable),
        reinterpret_cast<const char *>(&key));
    if (pEntry == 0) {
        return 0;
    }
    return reinterpret_cast<D>(Tcl_GetHashValue(pEntry));
}

template<class K, class D>
void
HashTable<K,D>::erase (const K &key)
{
    LOCK_MUTEX(m_mutex)

    Tcl_HashEntry *pEntry = Tcl_FindHashEntry(
        &m_hashTable,
        reinterpret_cast<const char *>(&key));
    if (pEntry != 0) {
        Tcl_DeleteHashEntry(pEntry);
    }
}

#endif
