// $Id: TclInterp.cpp,v 1.5 2000/04/28 19:37:53 chuang Exp $
#include <stdexcept>
#include <algorithm>
#include "RegistryKey.h"
#include "TclInterp.h"

TclInterp TclInterp::ms_instance;

TclInterp &
TclInterp::instance ()
{
    return ms_instance;
}

TclInterp::TclInterp ()
{
    // Look in registry for Tcl library to load.
    // Open Scriptics Tcl registry key.
    RegistryKey tclKey(HKEY_LOCAL_MACHINE, "SOFTWARE\\Scriptics\\Tcl");

    // Open any subkey under Scriptics Tcl key.
    std::string versionId = tclKey.subkeyName(0);
    RegistryKey versionKey(tclKey, versionId);

    // Get Tcl installation directory.
    m_rootDir = versionKey.value("Root");

    // Construct full path to Tcl library.
    std::string ver(
        versionId.begin(),
        std::remove(versionId.begin(), versionId.end(), '.'));
    std::string dllPath(m_rootDir);
    dllPath.append("\\bin\\tcl");
    dllPath.append(ver);
    dllPath.append(".dll");

    // Load Tcl library.
    _bstr_t path(dllPath.c_str());
    m_hmodTcl = CoLoadLibrary(path, TRUE);
    if (m_hmodTcl == NULL) {
        throw std::runtime_error("LoadLibrary");
    }

    // Get address of Tcl_CreateInterp function.
    typedef Tcl_Interp *(*CreateInterpFunc)();
    CreateInterpFunc createInterp = reinterpret_cast<CreateInterpFunc>(
        GetProcAddress(m_hmodTcl, "Tcl_CreateInterp"));
    if (createInterp == NULL) {
        throw std::runtime_error("GetProcAddress");
    }

    // Create Tcl interpreter.
    m_interp = createInterp();
    if (Tcl_InitStubs(m_interp, "8.1", 0) == NULL) {
        throw std::runtime_error("Tcl_InitStubs");
    }

    Tcl_FindExecutable(dllPath.c_str());

    // Execute Tcl_Init function.
    if (Tcl_Init(m_interp) != TCL_OK) {
        throw std::runtime_error("Tcl_Init");
    }
}

TclInterp::~TclInterp ()
{
    // TODO: I would like to clean up resources here, but when the destructor
    // is called on static instances of this class, the Tcl memory subsystem
    // may have been finalized, making the argument point to an invalid memory
    // address.
//    Tcl_DeleteInterp(m_interp);

    FreeLibrary(m_hmodTcl);
}

#if 0
void
TclInterp::initialize ()
{
    gs_pInstance = new TclInterp;
}

void
TclInterp::destroy ()
{
    delete gs_pInstance;
}
#endif
