// $Id: bindCmd.cpp,v 1.41 2000/03/29 16:27:29 chuang Exp $
#include "refCmd.h"
#include "bindCmd.h"

// This Tcl command binds a Tcl command to an event sink.

int
bindCmd (
    ClientData clientData,	/* Not used. */
    Tcl_Interp *interp,		/* Current interpreter. */
    int objc,			/* Number of arguments. */
    Tcl_Obj *CONST objv[])	/* The argument objects. */
{
    if (objc < 3 || objc > 4) {
	Tcl_WrongNumArgs(
            interp,
            1,
            objv,
            "object sinkCommand ?eventSinkIID?");
	return TCL_ERROR;
    }

    Reference *pReference =
        HandleSupport<Reference>::instance(interp).find(objv[1]);
    if (pReference == 0) {
        char *arg = Tcl_GetStringFromObj(objv[1], 0);
        Tcl_AppendResult(interp, "invalid interface pointer handle ", arg, NULL);
        return TCL_ERROR;
    }

    TclObject servant(objv[2]);

    char *sinkIIDStr = (objc < 4) ? 0 : Tcl_GetStringFromObj(objv[3], 0);

    try {
        HRESULT hr;

        IID sinkIID;
        if (sinkIIDStr != 0) {
            if (UuidFromString(reinterpret_cast<unsigned char *>(sinkIIDStr),
             &sinkIID) != RPC_S_OK) {
	        Tcl_AppendResult(
                    interp,
                    "cannot convert to IID: ",
                    sinkIIDStr,
                    NULL);
                return TCL_ERROR;
            }

        } else {
            IProvideClassInfo *pProvideClassInfo;
            hr = pReference->dispatch()->QueryInterface(
                IID_IProvideClassInfo,
                reinterpret_cast<void **>(&pProvideClassInfo));
            if (FAILED(hr)) {
                _com_issue_error(hr);
            }

            ITypeInfo *pClassTypeInfo;
            hr = pProvideClassInfo->GetClassInfo(&pClassTypeInfo);
            if (FAILED(hr)) {
                _com_issue_error(hr);
            }

            TypeInfo classTypeInfo(pClassTypeInfo);
            ITypeInfo *pSourceTypeInfo = classTypeInfo.getImplType(
                IMPLTYPEFLAG_FDEFAULT | IMPLTYPEFLAG_FSOURCE);
            if (pSourceTypeInfo == 0) {
                Tcl_AppendResult(
                    interp, "no default event source", NULL);
                return TCL_ERROR;
            }

            // Cache the source interface description.
            TypeInfo sourceTypeInfo(pSourceTypeInfo);
            TypeAttr sourceTypeAttr(sourceTypeInfo);
            InterfaceManager::instance().newInterface(
                sourceTypeAttr->guid, sourceTypeInfo);

            sinkIID = sourceTypeAttr->guid;
        }

        Interface *pSourceInterface = const_cast<Interface *>(
            InterfaceManager::instance().find(sinkIID));
        if (pSourceInterface == 0) {
            Tcl_AppendResult(
                interp, "no type information for sink interface", NULL);
            return TCL_ERROR;
        }

        pReference->advise(interp, *pSourceInterface, servant);
    }
    catch (_com_error &e) {
        return setComErrorResult(interp, e, __FILE__, __LINE__);
    }
    return TCL_OK;
}
