// $Id: importCmd.h,v 1.3 2000/03/27 16:49:21 chuang Exp $
#ifndef IMPORTCMD_H
#define IMPORTCMD_H

#include "TypeInfo.h"

// This represents a class of COM objects which can be created given a CLSID.

class ReferenceClass
{
    // CLSID of coclass
    CLSID m_clsid;

    // description of default interface for the class
    const Interface *m_pDefaultInterface;

    // description of source interface for the class
    const Interface *m_pSourceInterface;

public:
    ReferenceClass (REFCLSID clsid,
                    const Interface *pDefaultInterface,
                    const Interface *pSourceInterface):
        m_clsid(clsid),
        m_pDefaultInterface(pDefaultInterface),
        m_pSourceInterface(pSourceInterface)
    { }

    // Get class ID.
    const CLSID &clsid () const
    { return m_clsid; }

    // Get default interface description.
    const Interface *defaultInterface () const
    { return m_pDefaultInterface; }

    // Get source interface description.
    const Interface *sourceInterface () const
    { return m_pSourceInterface; }

    // Find instance by name.
    static const ReferenceClass *find(Tcl_Interp *interp, const char *pName);

    static const ReferenceClass *find (Tcl_Interp *interp, Tcl_Obj *pName)
    { return find(interp, Tcl_GetStringFromObj(pName, (int *)0)); }
};

// Get interface description given a command name.
extern const Interface *findInterfaceByCmdName(Tcl_Interp *interp,
                                               Tcl_Obj *pName);

extern int methodCmd(ClientData, Tcl_Interp *, int, Tcl_Obj *CONST []);
extern int propertyCmd(ClientData, Tcl_Interp *, int, Tcl_Obj *CONST []);
extern int interfaceCmd(ClientData, Tcl_Interp *, int, Tcl_Obj *CONST []);
extern int classCmd(ClientData, Tcl_Interp *, int, Tcl_Obj *CONST []);
extern int importCmd(ClientData, Tcl_Interp *, int, Tcl_Obj *CONST []);

#endif
